package cn.sh.stc.sict.cloud.common.gateway.filter;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import cn.sh.stc.sict.cloud.common.core.util.R;
import cn.sh.stc.sict.cloud.common.gateway.feign.RemoteAuthService;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.cloud.gateway.filter.GatewayFilter;
import org.springframework.cloud.gateway.filter.factory.AbstractGatewayFilterFactory;
import org.springframework.http.server.reactive.ServerHttpRequest;
import org.springframework.stereotype.Component;
import reactor.core.publisher.Mono;

/**
 * @author gao
 * @date 2023/08/30 10:23
 */
@Slf4j
@Component
@AllArgsConstructor
public class GptServerFilter extends AbstractGatewayFilterFactory {

    public final RemoteAuthService remoteAuthService;

    public static final String WEB_TOKEN = "ws-token";

    @Override
    public GatewayFilter apply(Object config) {
        return (exchange, chain) -> {
            ServerHttpRequest request = exchange.getRequest();
            String token = request.getQueryParams().getFirst(WEB_TOKEN);
            log.error("ws-token = {}", token);
            // 权限校验
            if (checkToken(token)) {
                return chain.filter(exchange);
            } else {
                return Mono.error(new RuntimeException("权限校验失败"));
            }
        };
    }

    /**
     * 校验token
     *
     * @param token
     * @return
     */
    private boolean checkToken(String token) {
        try {
            if (StrUtil.isBlank(token)) {
                return false;
            }

            R<Boolean> valid = remoteAuthService.isValid(token);
            if (ObjectUtil.isNull(valid) || ObjectUtil.isNull(valid.getData())) {
                return false;
            }
            return valid.getData();
        } catch (Exception e) {
            log.error("权限校验失败", e);
            return false;
        }
    }

}
